/*************************************************************************
* Rutoken                                                                *
* Copyright (c) 2003-2016, CJSC Aktiv-Soft. All rights reserved.         *
* Подробная информация:  http://www.rutoken.ru                           *
*------------------------------------------------------------------------*
* Данный пример демонстрирует удаление сертификата на токене с помощью   *
* pkicore. Для успешного выполнения программы на токене должен           *
* находиться по крайней мере один сертификат. В результате выполнения    *
* будет удален первый найденный сертификат.                              *
*************************************************************************/

#include <common.h>

using namespace std;
using namespace rutoken::pkicore;

int main()
{
	try {
		cout << boolalpha;

		/**********************************************************************
		* Инициализируем pkicore, передав путь до директории с библиотекой    *
		* rtPKCS11ECP.                                                        *
		**********************************************************************/
		rutoken::pkicore::initialize(".");
		SCOPE_EXIT() {
			/**********************************************************************
			* Завершаем работу с pkicore при выходе из текущего блока.            *
			**********************************************************************/
			rutoken::pkicore::deinitialize();
		};

		/**********************************************************************
		* Получаем список подключенных устройств и продолжаем работу с первым *
		* доступным устройством.                                              *
		**********************************************************************/
		auto devices = Pkcs11Device::enumerate();
		if (devices.empty()) {
			throw runtime_error("There must be at least one device connected");
		}

		auto device = move(devices.front());

		/**********************************************************************
		* Получаем список сертификатов на устройстве и продолжаем работу с    *
		* первым найденным.                                                   *
		**********************************************************************/
		auto certs = device.enumerateCerts();
		if (certs.empty()) {
			throw runtime_error("There must be at least one certificate on device");
		}

		auto cert = move(certs.front());

		/**********************************************************************
		* Авторизуемся на устройстве.                                         *
		**********************************************************************/
		device.login("12345678");
		SCOPE_EXIT(&device) {
			/**********************************************************************
			* Сбрасываем права доступа при выходе из текущего блока.              *
			**********************************************************************/
			device.logout();
		};

		/**********************************************************************
		* Удаляем сертификат.                                                 *
		**********************************************************************/
		device.deleteCert(cert);
	} catch (const exception& e) {
		cerr << e.what() << endl;
		return 1;
	}

	return 0;
}
