/*************************************************************************
* Rutoken                                                                *
* Copyright (c) 2003-2016, CJSC Aktiv-Soft. All rights reserved.         *
* Подробная информация:  http://www.rutoken.ru                           *
*------------------------------------------------------------------------*
* Данный пример демонстрирует импорт пользовательского сертификата на    *
* токен с помощью pkicore. Сертификат в формате DER читается из файла    *
* cert.cer. Для создания запроса на сертификацию можно воспользоваться   *
* примером Pkcs10Request.                                                *
*************************************************************************/

#include <common.h>

using namespace std;
using namespace rutoken::pkicore;

int main()
{
	try {
		cout << boolalpha;

		/**********************************************************************
		* Инициализируем pkicore, передав путь до директории с библиотекой    *
		* rtPKCS11ECP.                                                        *
		**********************************************************************/
		rutoken::pkicore::initialize(".");
		SCOPE_EXIT() {
			/**********************************************************************
			* Завершаем работу с pkicore при выходе из текущего блока.            *
			**********************************************************************/
			rutoken::pkicore::deinitialize();
		};

		/**********************************************************************
		* Получаем список подключенных устройств и продолжаем работу с первым *
		* доступным устройством.                                              *
		**********************************************************************/
		auto devices = Pkcs11Device::enumerate();
		if (devices.empty()) {
			throw runtime_error("There must be at least one device connected");
		}

		auto device = move(devices.front());

		/**********************************************************************
		* Авторизуемся на устройстве.                                         *
		**********************************************************************/
		device.login("12345678");
		SCOPE_EXIT(&device) {
			/**********************************************************************
			* Сбрасываем права доступа при выходе из текущего блока.              *
			**********************************************************************/
			device.logout();
		};

		/**********************************************************************
		* Читаем файл с сертификатом в DER формате.                           *
		**********************************************************************/
		const auto fileName = "cert.cer";

		cout << "Reading user certificate from " << fileName << endl;

		auto certData = readFile(fileName);

		/**********************************************************************
		* Импортируем пользовательский сертификат.                            *
		**********************************************************************/
		device.importUserCert(ExternalCert(certData.data(), certData.size()));
	} catch (const exception& e) {
		cerr << e.what() << endl;
		return 1;
	}

	return 0;
}
